
 // ***************************************************** // Starting of the code // ************************************************ //
// All the libraries needed //
#include <Arduino.h>
#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <TFT_eSPI.h>
#include <SPI.h>
// ********************** // You have to change this part // *********************** //
const char* ssid = "wifi"; //WiFi name
const char* password = "pass"; //WiFi password
const char* Gemini_Token = "Gemini API key"; //Gemini api key here
const char* Gemini_Max_Tokens = "100"; //more the tokens long the answer gets
String res = "";
 
// ******************** //                               // ************************ //
TFT_eSPI tft = TFT_eSPI();
// ******************************************* // Setup part of the code // ***************************************************** //
void setup() {
  Serial.begin(115200); //serial startup
  tft.init(); //display begin
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  tft.setRotation(1); // display in horizontal mode
  tft.fillScreen(TFT_BLACK); //display clear

  while (!Serial);
  // wait for WiFi connection
  WiFi.begin(ssid, password);
  Serial.print("Connecting to "); 
  tft.setTextColor(TFT_YELLOW);
  tft.setTextSize(2);
  tft.setCursor(0,0);
  tft.println("Connecting to ");
  Serial.println(ssid);
  tft.setCursor(10,20);
  tft.setTextColor(TFT_RED);
  tft.setTextSize(2);
  tft.println(ssid);
  tft.setTextColor(TFT_WHITE);
  tft.setTextSize(1);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.print(".");
    tft.println(".");

  }
  Serial.println("connected");
  tft.setTextColor(TFT_GREEN);
  tft.setTextSize(2);
  tft.println("Connected");
  Serial.print("IP address: "); //esp's ip address
  tft.fillScreen(TFT_CYAN);
  tft.setCursor(0,80);
  tft.setTextColor(TFT_MAGENTA);
  tft.setTextSize(2);
  tft.print("IP address: "); //esp's ip address
  tft.println(WiFi.localIP());
  Serial.println(WiFi.localIP());
  delay(1000);
}

// *********************************************** // End of setup part // **************************************************** //

void loop()  //main loop
{ // ********************************************* // Asking part (getting the question) // ****************************************************** //
  tft.fillScreen(TFT_BLACK); 
  tft.setTextColor(TFT_WHITE);
  tft.setCursor(0,0);
  tft.setTextSize(2);
  Serial.println("");
  tft.println(" Ask your Question : ");
  Serial.println("Ask your Question : ");
  while (!Serial.available())
    ;
  while (Serial.available()) {
    char add = Serial.read();
    res = res + add;
    delay(1);
  }
  int len = res.length();
  res = res.substring(0, (len - 1));
  res = "\"" + res + "\"";
  Serial.println("");
  Serial.print("Processing : ");
  Serial.println(res);
  tft.fillScreen(TFT_SKYBLUE);
  tft.setCursor(50,50);
  tft.setTextColor(TFT_BLUE);
  tft.setTextSize(3);
  tft.println("Processing :");
  // ************************************************ // Asking part end // *******************************************************************//


  // ***********************************************// Processing part (getting the answer from Gemini AI) *************************************//
  HTTPClient https;

  //Serial.print("[HTTPS] begin...\n");
  if (https.begin("https://generativelanguage.googleapis.com/v1beta/models/gemini-1.5-flash:generateContent?key=" + (String)Gemini_Token)) {  // HTTPS

    https.addHeader("Content-Type", "application/json");
    String payload = String("{\"contents\": [{\"parts\":[{\"text\":" + res + "}]}],\"generationConfig\": {\"maxOutputTokens\": " + (String)Gemini_Max_Tokens + "}}");

    //Serial.print("[HTTPS] GET...\n");

    // start connection and send HTTP header
    int httpCode = https.POST(payload);

    // httpCode will be negative on error
    // file found at server

    if (httpCode == HTTP_CODE_OK || httpCode == HTTP_CODE_MOVED_PERMANENTLY) {
      String payload = https.getString();
      //Serial.println(payload);

      DynamicJsonDocument doc(1024);


      deserializeJson(doc, payload);
      String Answer = doc["candidates"][0]["content"]["parts"][0]["text"];


      
      // For Filtering our Special Characters, WhiteSpaces and NewLine Characters
      Answer.trim();
      String filteredAnswer = "";
      for (size_t i = 0; i < Answer.length(); i++) {
        char c = Answer[i];
        if (isalnum(c) || isspace(c)) {
          filteredAnswer += c;
        } else {
          filteredAnswer += ' ';
        }
      }
      Answer = filteredAnswer;
      tft.fillScreen(TFT_BLACK);
      tft.setCursor(5,5);
      tft.setTextColor(TFT_YELLOW);
      Serial.println("");
      tft.println("Here is your Answer:");
      Serial.println("Here is your Answer: ");
      Serial.println("");
      tft.fillScreen(TFT_BLACK);
      tft.setTextColor(TFT_WHITE);
      tft.setTextSize(1);
      tft.setCursor(0,0);
      Serial.println(Answer);
      tft.println(Answer);
      delay(10000);
   // ************************************************************** // End of processing part // *************************************************** //

   // ************************************************************** // In case of an error // ****************************************************** //
    } else {
      Serial.printf("[HTTPS] GET... failed, error: %s\n", https.errorToString(httpCode).c_str());
      tft.fillScreen(TFT_SKYBLUE);
      tft.setCursor(50,50);
      tft.setTextColor(TFT_RED);
      tft.setTextSize(2);
      tft.println("error!");
    }
    https.end();
  } else {
    tft.fillScreen(TFT_SKYBLUE);
    tft.setCursor(50,50);
    tft.setTextColor(TFT_RED);
    tft.setTextSize(2);
    tft.println("error!");
    Serial.printf("[HTTPS] Unable to connect\n");
  }
 // ************************************************************* // Ending of error part // ******************************************************* //
  res = "";
 // ************************************************************* // End of the code // ************************************************************ //
}